using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

using GameMenu;

namespace GameMenuSample
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        Menu mainMenu;
        KeyboardState prevKeyBoardState;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            mainMenu = new GameMenu.Menu(this);
            mainMenu.textColor = Color.Pink;
            mainMenu.selectColor = Color.Orange;

            mainMenu.AddChoice("Choice 1");
            MenuChoice choice = mainMenu.AddChoice("LR Choice");
            choice.AddLeftRightChoices(new string[] { "one", "two", "three" });

            choice = mainMenu.AddChoice("Sub Menu");
            choice.AddChoice("Sub choice 1");
            choice.AddChoice("Sub choice 2");

            mainMenu.ChoiceExecuted += new Menu.ChoiceExecutedHandler(ChoiceExecuted);
            mainMenu.ChoiceSelected += new Menu.ChoiceSelectedHandler(ChoiceSelected);
            mainMenu.ChoiceDeselected += new Menu.ChoiceDeselectedHandler(ChoiceDeselected);
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        public void ChoiceExecuted(object source, Menu.MenuEvent e)
        {
            Console.WriteLine("Executed: " + e.choiceString);

            if (e.choiceString == "Return")
                mainMenu.visible = false;
        }

        public void ChoiceSelected(object source, Menu.MenuEvent e)
        {
            Console.WriteLine("Selected: " + e.choiceString);
        }

        public void ChoiceDeselected(object source, Menu.MenuEvent e)
        {
            Console.WriteLine("Deselected: " + e.choiceString);
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            KeyboardState keyboardState = Keyboard.GetState();

            if (keyboardState.IsKeyDown(Keys.Escape) && !prevKeyBoardState.IsKeyDown(Keys.Escape))
            {
                mainMenu.visible = !mainMenu.visible;
            }

            mainMenu.Update(gameTime);


            prevKeyBoardState = keyboardState;
            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

            mainMenu.Draw(gameTime);

            base.Draw(gameTime);
        }
    }
}
